@@ -1,39 +1,17 @@
 import os
 import dlchord
+import itertools
 from dlchord import Chord
 from pywfd import label as lb
+from pywfd import const
 
-chord_quality = {
-    0: "",
-    1: "m",
-    2: "dim7",
-    3: "aug",
-    4: "6",
-    5: "7",
-    6: "M7",
-    7: "add9",
-    8: "m6",
-    9: "m7",
-    10: "mM7",
-    11: "sus4",
-    12: "7sus4",
-    13: "m7-5"}
-chord_tone = {
-    0: "C",
-    1: "C#",
-    2: "D",
-    3: "D#",
-    4: "E",
-    5: "F",
-    6: "F#",
-    7: "G",
-    8: "G#",
-    9: "A",
-    10: "A#",
-    11: "B"}
-
-tones = ["C", "Db", "D", "Eb", "E", "F", "Gb", "G", "Ab", "A", "Bb", "B", "N"]
 
+def ToMajor(key):
+    if key == const.KEYLENGTH:
+        return -1
+    if key > 11:
+        return key - 12
+    return key
 
 def splitindex(l, n):
     for idx in range(0, len(l), n):
@@ -53,10 +31,10 @@ def number_to_chord(chord_num, is_input=False, on_chord=False):
         name = (chord_num - 11) // 12
         pitch += 1
 
-    pitch %= 12
-    name %= 14
+    pitch %= len(const.CHORD_TONE)
+    name %= len(const.CHORD_QUALITY)
 
-    return chord_tone[pitch] + chord_quality[name]
+    return const.CHORD_TONE[pitch] + const.CHORD_QUALITY[name]
 
 
 def chord_to_array(chord, tension=True):
@@ -82,24 +60,24 @@ def chord_to_array(chord, tension=True):
         if chord.bass == chord.root:
             if "b9" in quality:
                 if "m" in quality:
-                    chord = Chord(Chord(tones[chord.root] + "7").transpose(3).chord + "/" + tones[chord.root])
+                    chord = Chord(Chord(const.TONES[chord.root] + "7").transpose(3).chord + "/" + const.TONES[chord.root])
                 else:
-                    chord = Chord(Chord(tones[chord.root] + "dim7").transpose(4).chord + "/" + tones[chord.root])
+                    chord = Chord(Chord(const.TONES[chord.root] + "dim7").transpose(4).chord + "/" + const.TONES[chord.root])
             elif "9" in quality and "add" not in quality and "sus" not in quality and "#9" not in quality:
                 if "m" in quality and "M" not in quality:
-                    chord = Chord(Chord(tones[chord.root] + "M7").transpose(3).chord + "/" + tones[chord.root])
+                    chord = Chord(Chord(const.TONES[chord.root] + "M7").transpose(3).chord + "/" + const.TONES[chord.root])
                 elif "M" in quality:
-                    chord = Chord(Chord(tones[chord.root] + "m7").transpose(4).chord + "/" + tones[chord.root])
+                    chord = Chord(Chord(const.TONES[chord.root] + "m7").transpose(4).chord + "/" + const.TONES[chord.root])
                 else:
-                    chord = Chord(Chord(tones[chord.root] + "m7-5").transpose(4).chord + "/" + tones[chord.root])
+                    chord = Chord(Chord(const.TONES[chord.root] + "m7-5").transpose(4).chord + "/" + const.TONES[chord.root])
             elif "11" in quality:
                 if "m" in quality:
-                    chord = Chord(Chord(tones[chord.root] + "add9").transpose(3).chord + "/" + tones[chord.root])
+                    chord = Chord(Chord(const.TONES[chord.root] + "add9").transpose(3).chord + "/" + const.TONES[chord.root])
 
     quality = chord.quality.quality
     chord_num += chord.root
 
-    sorted_quality = sorted(chord_quality.items(), key=lambda x: len(x[1]), reverse=True)
+    sorted_quality = sorted(const.CHORD_QUALITY.items(), key=lambda x: len(x[1]), reverse=True)
     for idx, qua in sorted_quality:
         if qua[:3] in quality:
             chord_num += (12 * idx)
@@ -139,12 +117,23 @@ def label_to_chord(label, sep=':'):
 
 def is_wavetone_chord(chord):
     tension = ["9", "13", "11", "M7-5", "aug7", "augM7"]
-    if any([t in chord for t in tension]):
+    if any([t in chord for t in tension]) and "add9" not in chord:
         return False
     else:
         return True
 
 
+def getIndexNearestTime(times, time):
+    near = -1
+    index = 0
+    for i, t in enumerate(times):
+        if abs(t[0] - time) < near or near == -1:
+            near = abs(t[0] - time)
+            index = i
+
+    return index
+
+
 class ChordSplit:
     def __init__(self, chord, rhythm, label):
         self._raw_chord = chord
@@ -166,13 +155,16 @@ class ChordSplit:
         chord = []
         for i in range(len(raw_chord)):
             parse_chord = self._split(i)
-            if parse_chord != "" and parse_chord != "N.C." and self.rhythm.musickey(i) != 12:
-                parse_chord = dlchord.Chord(parse_chord).modify(tones[self.rhythm.musickey(i)], advanced=advanced).chord
+            if parse_chord != "" and parse_chord != "N.C.":
+                key = ToMajor(self.rhythm.musickey(i))
+                if key == -1:
+                    key = 0
+                parse_chord = dlchord.Chord(parse_chord).modify(const.KEY_TONES[key], advanced=advanced).chord
             chord.append(parse_chord)
 
         return chord
 
-    def chordLabel(self, advanced=False):
+    def chordLabel(self, advanced=False, allow_label=True):
         """[summary]
         
         Returns:
@@ -184,21 +176,70 @@ class ChordSplit:
 
         chord = ""
         now_chord = ""
-        for i in range(len(self.chord)):
-            now_chord = self.chord[i]
+        for i, now_chord in enumerate(self.chord):
 
             if now_chord != '' and now_chord != "N.C.":
-                now_chord = dlchord.Chord(now_chord).modify(tones[self.rhythm.musickey(i)], advanced=advanced).chord
+                key = ToMajor(self.rhythm.musickey(i))
+                if key == -1:
+                    key = 0
+                now_chord = dlchord.Chord(now_chord).modify(const.KEY_TONES[key], advanced=advanced).chord
 
             if i == 0:
                 chord = now_chord
 
-            if chord != now_chord and now_chord != '' or i == (len(self.chord) - 1):
-                e_time = self.rhythm.time(i)
+            if (chord != now_chord and now_chord != '') or i == len(self.chord) - 1:
+                s_time = round(s_time, 4)
+                e_time = round(self.rhythm.time(i), 4)
                 times.append([s_time, e_time, chord])
                 chord = now_chord
                 s_time = e_time
 
+        if allow_label:
+            for label in self.label.label_list:
+                label_value = label.getLabel()
+                if label_value[:1] == "!":
+                    label_time = label.getTime() / 1000
+                    index = getIndexNearestTime(times, label_time)
+                    times[index][-1] = label_value[1:]
+
+        return times
+
+    def sectionLabel(self):
+        times = []
+        labels = []
+        for label in self.label.label_list:
+            label_value = label.getLabel()
+            if label_value[:2].lower() == "s!":
+                label_time = label.getTime() / 1000
+                labels.append([label_time, label_value[2:]])
+
+        if labels:
+            label_len = len(labels)
+            for i in range(label_len):
+                if len(labels) < i + 2:
+                    times.append([labels[i][0], round(self.rhythm.length, 4), labels[i][-1]])
+                else:
+                    times.append([labels[i][0], labels[i + 1][0], labels[i][-1]])
+
+        return times
+
+    def bpmLabel(self):
+        times = []
+        labels = []
+        for label in self.label.label_list:
+            label_value = label.getLabel()
+            if label_value[:2].lower() == "b!":
+                label_time = label.getTime() / 1000
+                labels.append([label_time, label_value[2:]])
+
+        if labels:
+            label_len = len(labels)
+            for i in range(label_len):
+                if len(labels) < i + 2:
+                    times.append([labels[i][0], round(self.rhythm.length, 4), labels[i][-1]])
+                else:
+                    times.append([labels[i][0], labels[i + 1][0], labels[i][-1]])
+
         return times
 
     def keyLabel(self):
@@ -217,7 +258,7 @@ class ChordSplit:
 
             if key != now_key or i == (len(self.rhythm.rhythmkey.rhythm_key_map) - 1):
                 e_time = self.rhythm.time(i)
-                times.append([s_time, e_time, tones[key]])
+                times.append([s_time, e_time, const.KEY_TONES[key]])
                 key = now_key
                 s_time = e_time
 
@@ -239,7 +280,7 @@ class ChordSplit:
 
         if min_time - label[0][0] > duration:
             label[0][0] += abs(min_time - label[0][0])
-
+        
         for i in range(len(self.chord)):
             if len(label) <= offset:
                 break
@@ -247,19 +288,19 @@ class ChordSplit:
             duration = abs(self.rhythm.time(i) - self.rhythm.time(i + 1))
 
             dist = abs(self.rhythm.time(i) - label[offset][0])
-            if dist < duration:
-                if dist <= abs(self.rhythm.time(i + 1) - label[offset][0]):
-                    chordlist.append(chord_to_array(label[offset][-1]))
-                    if not is_wavetone_chord(label[offset][-1]):
-                        c_label = lb.LabelSplit()
-                        c_label.setTime(int(self.rhythm.time(i) * 1000))
-                        c_label.setLabel(label[offset][-1])
-                        self.label.append(c_label)
-                    offset += 1
+            if dist < duration and dist <= abs(self.rhythm.time(i + 1) - label[offset][0]):
+                chordlist.append(chord_to_array(label[offset][-1]))
+                if not is_wavetone_chord(label[offset][-1]):
+                    c_label = lb.LabelSplit()
+                    c_label.setTime(int(self.rhythm.time(i) * 1000))
+                    c_label.setLabel("!" + label[offset][-1])
+                    self.label.append(c_label)
+                offset += 1
+            else:
+                if i == 0:
+                    chordlist.append(chord_to_array("N.C."))
                 else:
                     chordlist.append(chord_to_array(''))
-            else:
-                chordlist.append(chord_to_array(''))
 
         return chordlist
 
@@ -272,8 +313,9 @@ class ChordSplit:
         for i, chord in enumerate(self.chord, start=1):
             if form:
                 try:
-                    if self.rhythm.musickey(i - 1) != 12:
-                        chord = dlchord.Chord(chord).modify(tones[self.rhythm.musickey(i - 1)], advanced=advanced)
+                    key = ToMajor(self.rhythm.musickey(i - 1))
+                    if key != -1:
+                        chord = dlchord.Chord(chord).modify(const.KEY_TONES[key], advanced=advanced)
                 except ValueError:
                     pass
                 
